-- **************************************************************************
-- * JellyTitan.lua
-- *
-- * By: Smashed
-- *
-- **************************************************************************

-- ******************************** Constants *******************************
local _G = getfenv(0);
local TITAN_JELLY_ID = "Jelly";
local TITAN_JELLY_THRESHOLD_TABLE = {
     Values = { 0.5, 0.75 },
     Colors = { GREEN_FONT_COLOR, NORMAL_FONT_COLOR, RED_FONT_COLOR },
}
local updateTable = {TITAN_JELLY_ID, TITAN_PANEL_UPDATE_BUTTON} ;
-- ******************************** Variables *******************************
local L = LibStub("AceLocale-3.0"):GetLocale("Titan", true)
local AceTimer = LibStub("AceTimer-3.0")
local JellyTimer
-- ******************************** Functions *******************************

-- **************************************************************************
-- NAME : TitanPanelJellyButton_OnLoad()
-- DESC : Registers the plugin upon it loading
-- **************************************************************************
function TitanPanelJellyButton_OnLoad(self)
	self.registry = {
		id = TITAN_JELLY_ID,
		--          builtIn = 1,
		category = "Built-ins",
		version = TITAN_VERSION,
		menuText = L["TITAN_JELLY_MENU_TEXT"],
		buttonTextFunction = "TitanPanelJellyButton_GetButtonText",
		tooltipTitle = L["TITAN_JELLY_TOOLTIP"],
		tooltipTextFunction = "TitanPanelJellyButton_GetTooltipText",
		icon = "Interface\\AddOns\\Jelly\\JellyTitan",
		iconWidth = 16,
		controlVariables = {
			ShowIcon = true,
			ShowLabelText = true,
			ShowRegularText = false,
			ShowColoredText = true,
			DisplayOnRightSide = false
		},
		savedVariables = {
			ShowUsedSlots = 1,
			ShowDetailedInfo = false,
			CountAmmoPouchSlots = false,
			CountShardJellySlots = false,
			CountProfJellySlots = false,
			ShowIcon = 1,
			ShowLabelText = 1,
			ShowColoredText = 1,
		}
	};

	self:RegisterEvent("PLAYER_ENTERING_WORLD");
end

-- **************************************************************************
-- NAME : TitanPanelJellyButton_OnEvent()
-- DESC : Parse events registered to plugin and act on them
-- **************************************************************************
function TitanPanelJellyButton_OnEvent(self, event, ...)
	if (event == "PLAYER_ENTERING_WORLD") and (not self:IsEventRegistered("JELLY_UPDATE")) then
		self:RegisterEvent("JELLY_UPDATE");
	end

	if event == "JELLY_UPDATE" then
		-- Create the only when the event is active
		self:SetScript("OnUpdate", TitanPanelJellyButton_OnUpdate)
	end
--[[
	if event == "JELLY_UPDATE" then
		-- Throw a timer to update the button 3 seconds after the initial event to avoid "spammy" events
		if not JellyTimer then
			JellyTimer = AceTimer.ScheduleTimer("TitanPanelJelly", TitanPanelJellyButton_OnUpdate, 3, updateTable)
		end
	end
--]]
end

function TitanPanelJellyButton_OnUpdate(self)
	-- update the button
	TitanPanelPluginHandle_OnUpdate(updateTable)
	-- remove until the next Jelly event
	self:SetScript("OnUpdate", nil)
end
--[[
function TitanPanelJellyButton_OnUpdate(table)
	TitanPanelPluginHandle_OnUpdate(table)
	AceTimer.CancelAllTimers("TitanPanelJelly")
	JellyTimer = nil;
end
--]]
-- **************************************************************************
-- NAME : TitanPanelJellyButton_OnClick(button)
-- DESC : Opens all Jellys on a LeftClick
-- VARS : button = value of action
-- **************************************************************************
function TitanPanelJellyButton_OnClick(self, button)
	if (button == "LeftButton") then
		OpenAllJellys();
	end
end

-- **************************************************************************
-- NAME : TitanPanelJellyButton_GetButtonText(id)
-- DESC : Calculate Jelly space logic then display data on button
-- VARS : id = button ID
-- **************************************************************************
function TitanPanelJellyButton_GetButtonText(id)
	local button, id = TitanUtils_GetButton(id, true);
	local totalSlots, usedSlots, availableSlots;
	local useme;

	totalSlots = 0;
	usedSlots = 0;
	for Jelly = 0, 4 do
		if TitanGetVar(TITAN_JELLY_ID, "CountAmmoPouchSlots") and TitanJELLY_IsAmmoPouch(GetJellyName(Jelly)) then
			useme = 1;
		elseif TitanGetVar(TITAN_JELLY_ID, "CountShardJellySlots") and TitanJELLY_IsShardJelly(GetJellyName(Jelly)) then
			useme = 1;
		elseif TitanGetVar(TITAN_JELLY_ID, "CountProfJellySlots") and TitanJELLY_IsProfJelly(GetJellyName(Jelly)) then
			useme = 1;
		elseif not TitanJELLY_IsAmmoPouch(GetJellyName(Jelly)) and not TitanJELLY_IsShardJelly(GetJellyName(Jelly))
			and not TitanJELLY_IsProfJelly(GetJellyName(Jelly)) then
			useme = 1;
		else
			useme = 0;
		end

		if useme == 1 then
			local size = GetContainerNumSlots(Jelly);
			if (size and size > 0) then
				totalSlots = totalSlots + size;
				for slot = 1, size do
					if (GetContainerItemInfo(Jelly, slot)) then
						usedSlots = usedSlots + 1;
					end
				end
			end
		end
	end
	availableSlots = totalSlots - usedSlots;

	local JellyText, JellyRichText, color;
	if (TitanGetVar(TITAN_JELLY_ID, "ShowUsedSlots")) then
		JellyText = format(L["TITAN_JELLY_FORMAT"], usedSlots, totalSlots);
	else
		JellyText = format(L["TITAN_JELLY_FORMAT"], availableSlots, totalSlots);
	end

	if ( TitanGetVar(TITAN_JELLY_ID, "ShowColoredText") ) then
		color = TitanUtils_GetThresholdColor(TITAN_JELLY_THRESHOLD_TABLE, usedSlots / totalSlots);
		JellyRichText = TitanUtils_GetColoredText(JellyText, color);
	else
		JellyRichText = TitanUtils_GetHighlightText(JellyText);
	end

	return L["TITAN_JELLY_BUTTON_LABEL"], JellyRichText;
end

-- **************************************************************************
-- NAME : TitanPanelJellyButton_GetTooltipText()
-- DESC : Display tooltip text
-- **************************************************************************
function TitanPanelJellyButton_GetTooltipText()
	local totalSlots, usedSlots, availableSlots;
	local returnstring = "";

	if TitanGetVar(TITAN_JELLY_ID, "ShowDetailedInfo") then
		returnstring = "\n";
		if TitanGetVar(TITAN_JELLY_ID, "ShowUsedSlots") then
			returnstring = returnstring..TitanUtils_GetNormalText(L["TITAN_JELLY_MENU_TEXT"])
				..":\t"..TitanUtils_GetNormalText(L["TITAN_JELLY_USED_SLOTS"])..":\n";
		else
			returnstring = returnstring..TitanUtils_GetNormalText(L["TITAN_JELLY_MENU_TEXT"])
				..":\t"..TitanUtils_GetNormalText(L["TITAN_JELLY_FREE_SLOTS"])..":\n";
		end

		for Jelly = 0, 4 do
			totalSlots = GetContainerNumSlots(Jelly) or 0;
			availableSlots = GetContainerNumFreeSlots(Jelly) or 0;
			usedSlots = totalSlots - availableSlots;
			local itemlink  = Jelly > 0 and GetInventoryItemLink("player", ContainerIDToInventoryID(Jelly))
				or TitanUtils_GetHighlightText(L["TITAN_JELLY_BACKPACK"]).. FONT_COLOR_CODE_CLOSE;


			if itemlink then
				itemlink = string.gsub( itemlink, "%[", "" );
				itemlink = string.gsub( itemlink, "%]", "" );
			end

			if Jelly > 0 and not GetInventoryItemLink("player", ContainerIDToInventoryID(Jelly)) then
				itemlink = nil;
			end

			local JellyText, JellyRichText, color;
			if (TitanGetVar(TITAN_JELLY_ID, "ShowUsedSlots")) then
				JellyText = format(L["TITAN_JELLY_FORMAT"], usedSlots, totalSlots);
			else
				JellyText = format(L["TITAN_JELLY_FORMAT"], availableSlots, totalSlots);
			end

			if ( TitanGetVar(TITAN_JELLY_ID, "ShowColoredText") ) then
				if totalSlots == 0 then
					color = TitanUtils_GetThresholdColor(TITAN_JELLY_THRESHOLD_TABLE, 1 );
				else
					color = TitanUtils_GetThresholdColor(TITAN_JELLY_THRESHOLD_TABLE, usedSlots / totalSlots);
				end
				JellyRichText = TitanUtils_GetColoredText(JellyText, color);
			else
				JellyRichText = TitanUtils_GetHighlightText(JellyText);
			end

			if itemlink then
				returnstring = returnstring..itemlink.."\t"..JellyRichText.."\n";
			end
		end
		returnstring = returnstring.."\n";
	end
	return returnstring..TitanUtils_GetGreenText(L["TITAN_JELLY_TOOLTIP_HINTS"]);
end

-- **************************************************************************
-- NAME : TitanPanelRightClickMenu_PrepareJellyMenu()
-- DESC : Display rightclick menu options
-- **************************************************************************
function TitanPanelRightClickMenu_PrepareJellyMenu()
		 local info

		 -- level 2
	if _G["UIDROPDOWNMENU_MENU_LEVEL"] == 2 then
		if _G["UIDROPDOWNMENU_MENU_VALUE"] == "Options" then
			TitanPanelRightClickMenu_AddTitle(L["TITAN_PANEL_MENU_OPTIONS"], _G["UIDROPDOWNMENU_MENU_LEVEL"])
			info = {};
			info.text = L["TITAN_JELLY_MENU_SHOW_USED_SLOTS"];
			info.func = TitanPanelJellyButton_ShowUsedSlots;
			info.checked = TitanGetVar(TITAN_JELLY_ID, "ShowUsedSlots");
			UIDropDownMenu_AddButton(info, _G["UIDROPDOWNMENU_MENU_LEVEL"]);

			info = {};
			info.text = L["TITAN_JELLY_MENU_SHOW_AVAILABLE_SLOTS"];
			info.func = TitanPanelJellyButton_ShowAvailableSlots;
			info.checked = TitanUtils_Toggle(TitanGetVar(TITAN_JELLY_ID, "ShowUsedSlots"));
			UIDropDownMenu_AddButton(info, _G["UIDROPDOWNMENU_MENU_LEVEL"]);

			info = {};
			info.text = L["TITAN_JELLY_MENU_SHOW_DETAILED"];
			info.func = TitanPanelJellyButton_ShowDetailedInfo;
			info.checked = TitanGetVar(TITAN_JELLY_ID, "ShowDetailedInfo");
			UIDropDownMenu_AddButton(info, _G["UIDROPDOWNMENU_MENU_LEVEL"]);
		end
		if _G["UIDROPDOWNMENU_MENU_VALUE"] == "IgnoreCont" then
			TitanPanelRightClickMenu_AddTitle(L["TITAN_JELLY_MENU_IGNORE_SLOTS"], _G["UIDROPDOWNMENU_MENU_LEVEL"])
			info = {};
			info.text = L["TITAN_JELLY_MENU_IGNORE_AMMO_POUCH_SLOTS"];
			info.func = TitanPanelJellyButton_ToggleIgnoreAmmoPouchSlots;
			info.checked = TitanUtils_Toggle(TitanGetVar(TITAN_JELLY_ID, "CountAmmoPouchSlots"));
			UIDropDownMenu_AddButton(info, _G["UIDROPDOWNMENU_MENU_LEVEL"]);

			info = {};
			info.text = L["TITAN_JELLY_MENU_IGNORE_SHARD_JellyS_SLOTS"];
			info.func = TitanPanelJellyButton_ToggleIgnoreShardJellySlots;
			info.checked = TitanUtils_Toggle(TitanGetVar(TITAN_JELLY_ID, "CountShardJellySlots"));
			UIDropDownMenu_AddButton(info, _G["UIDROPDOWNMENU_MENU_LEVEL"]);

			info = {};
			info.text = L["TITAN_JELLY_MENU_IGNORE_PROF_JellyS_SLOTS"];
			info.func = TitanPanelJellyButton_ToggleIgnoreProfJellySlots;
			info.checked = TitanUtils_Toggle(TitanGetVar(TITAN_JELLY_ID, "CountProfJellySlots"));
			UIDropDownMenu_AddButton(info, _G["UIDROPDOWNMENU_MENU_LEVEL"]);
		end
		return
	end

	-- level 1
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_JELLY_ID].menuText);

	info = {};
	info.text = L["TITAN_PANEL_MENU_OPTIONS"];
	info.value = "Options"
	info.hasArrow = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.text = L["TITAN_JELLY_MENU_IGNORE_SLOTS"];
	info.value = "IgnoreCont"
	info.hasArrow = 1;
	UIDropDownMenu_AddButton(info);

	TitanPanelRightClickMenu_AddSpacer();
	TitanPanelRightClickMenu_AddToggleIcon(TITAN_JELLY_ID);
	TitanPanelRightClickMenu_AddToggleLabelText(TITAN_JELLY_ID);
	TitanPanelRightClickMenu_AddToggleColoredText(TITAN_JELLY_ID);
	TitanPanelRightClickMenu_AddSpacer();
	TitanPanelRightClickMenu_AddCommand(L["TITAN_PANEL_MENU_HIDE"], TITAN_JELLY_ID, TITAN_PANEL_MENU_FUNC_HIDE);
end

-- **************************************************************************
-- NAME : TitanPanelJellyButton_ShowUsedSlots()
-- DESC : Set option to show used slots
-- **************************************************************************
function TitanPanelJellyButton_ShowUsedSlots()
	TitanSetVar(TITAN_JELLY_ID, "ShowUsedSlots", 1);
	TitanPanelButton_UpdateButton(TITAN_JELLY_ID);
end

-- **************************************************************************
-- NAME : TitanPanelJellyButton_ShowAvailableSlots()
-- DESC : Set option to show available slots
-- **************************************************************************
function TitanPanelJellyButton_ShowAvailableSlots()
	TitanSetVar(TITAN_JELLY_ID, "ShowUsedSlots", nil);
	TitanPanelButton_UpdateButton(TITAN_JELLY_ID);
end

-- **************************************************************************
-- NAME : TitanPanelJellyButton_ToggleIgnoreAmmoPouchSlots()
-- DESC : Set option to count ammo pouch slots
-- **************************************************************************
function TitanPanelJellyButton_ToggleIgnoreAmmoPouchSlots()
	TitanToggleVar(TITAN_JELLY_ID, "CountAmmoPouchSlots");
	TitanPanelButton_UpdateButton(TITAN_JELLY_ID);
end

-- **************************************************************************
-- NAME : TitanPanelJellyButton_ToggleIgnoreShardJellySlots()
-- DESC : Set option to count shard Jelly slots
-- **************************************************************************
function TitanPanelJellyButton_ToggleIgnoreShardJellySlots()
	TitanToggleVar(TITAN_JELLY_ID, "CountShardJellySlots");
	TitanPanelButton_UpdateButton(TITAN_JELLY_ID);
end

-- **************************************************************************
-- NAME : TitanPanelJellyButton_ToggleIgnoreProfJellySlots()
-- DESC : Set option to count profession Jelly slots
-- **************************************************************************
function TitanPanelJellyButton_ToggleIgnoreProfJellySlots()
	TitanToggleVar(TITAN_JELLY_ID, "CountProfJellySlots");
	TitanPanelButton_UpdateButton(TITAN_JELLY_ID);
end


function TitanPanelJellyButton_ShowDetailedInfo()
	TitanToggleVar(TITAN_JELLY_ID, "ShowDetailedInfo");
end

-- **************************************************************************
-- NAME : TitanJELLY_IsAmmoPouch(name)
-- DESC : Test to see if Jelly is an ammo pouch
-- VARS : name = item name
-- **************************************************************************
function TitanJELLY_IsAmmoPouch(name)
	if (name) then
		for index, value in pairs(L["TITAN_JELLY_AMMO_POUCH_NAMES"]) do
			if (string.find(name, value)) then
				return true;
			end
		end
	end
	return false;
end

-- **************************************************************************
-- NAME : TitanJELLY_IsShardJelly(name)
-- DESC : Test to see if Jelly is a shard Jelly
-- VARS : name = item name
-- **************************************************************************
function TitanJELLY_IsShardJelly(name)
	if (name) then
		for index, value in pairs(L["TITAN_JELLY_SHARD_JELLY_NAMES"]) do
			if (string.find(name, value)) then
				return true;
			end
		end
	end
	return false;
end

-- **************************************************************************
-- NAME : TitanJELLY_IsProfJelly(name)
-- DESC : Test to see if Jelly is a profession Jelly
-- VARS : name = item name
-- **************************************************************************
function TitanJELLY_IsProfJelly(name)
	if (name) then
		for index, value in pairs(L["TITAN_JELLY_PROF_JELLY_NAMES"]) do
			if (string.find(name, value, 1, true)) then
				return true;
			end
		end
	end
	return false;
end
